/*******************************************************************************
 * Copyright 2011-2012, Cypress Semiconductor Corporation.  All rights reserved.
 * You may use this file only in accordance with the license, terms, conditions,
 * disclaimers, and limitations in the end user license agreement accompanying
 * the software package with which this file was provided.
 ********************************************************************************/

#include <string.h>
#include <stdint.h>
#include "cybtldr_parse.h"
#include "cybtldr_command.h"
#include "cybtldr_api.h"
#include "cybtldr_api2.h"
#include "Uart.h"

uint8_t g_abort;    // Global Flag for starting and abortiing the Bootloader Communication between Air chip and Touch Panel

int ProcessDataRow_v0(CyBtldr_Action action, uint32_t rowSize, uint8_t *rowData, CyBtldr_ProgressUpdate *update)
{
    uint8_t buffer[MAX_BUFFER_SIZE];
    uint16_t bufSize;
    uint8_t arrayId;
    uint16_t rowNum;
    uint8_t checksum;

    int err;

    err = CyBtldr_ParseRowData(rowSize, rowData, &arrayId, &rowNum, buffer, &bufSize, &checksum);
    if (CYRET_SUCCESS == err)
    {
        switch (action)
        {
        case ERASE:
            err = CyBtldr_EraseRow(arrayId, rowNum);
            break;
        case PROGRAM:
            err = CyBtldr_ProgramRow(arrayId, rowNum, buffer, bufSize);
            if (CYRET_SUCCESS != err)
                break;
            /* Continue on to verify the row that was programmed */
        case VERIFY:
            checksum = (uint8_t)(checksum + arrayId + rowNum + (rowNum >> 8) + bufSize + (bufSize >> 8));
            err = CyBtldr_VerifyRow(arrayId, rowNum, checksum);
            break;
        } 
    }

    /*Jab jab Ek Row flash ho jayega tab tab hum log ise call kar k arrayID or rowNum isme save kar lenge taaki 
    next time me jab bhi aau doosra row flash kar k to iske pass data hoga ki bhai aapne pehle hi ek row flash kiya he 
    ab ye dusra bhi ho gya he jise ye update kar dega. 
    
    err = CyBtldr_CreateEraseRowCmd(arrayID, rowNum, inBuf, &inSize, &outSize); ye function sirf ek baar me ek row hi 
    flash ya Erase kar sakta he. to ye har baar aa k data(arrayID and rowNumber) save karta jayega.*/ 

    if (CYRET_SUCCESS == err && NULL != update)
        update(arrayId, rowNum);    // Here we re actuallly calling the function pointer.

    return err;
}

// This RunAction_v0() returns the Error Code if the Header is Parsed Sucessfully.
int RunAction_v0(CyBtldr_Action action, uint32_t lineLen, uint8_t *line, uint8_t appId,
                 const uint8_t *securityKey, CyBtldr_ProgressUpdate *update)
{
    const uint8_t INVALID_APP = 0xFF;
    uint32_t blVer = 0; // Bootloader Version from the Botloader component in PSOC Creator
    uint32_t siliconId = 0;
    uint8_t siliconRev = 0;
    uint8_t chksumtype = SUM_CHECKSUM;
    uint8_t isValid;
    uint8_t isActive;
    int err;
    uint8_t bootloaderEntered = 0;

    // Isko humne Silicon ID, SiliconRev or Checksum nikaalne k liye call kiya he.
    err = CyBtldr_ParseHeader(lineLen, line, &siliconId, &siliconRev, &chksumtype);

    if (CYRET_SUCCESS == err)
    {
        CyBtldr_SetCheckSumType(chksumtype);

        printf("Starting Bootloader Operation Now\n");
        err = CyBtldr_StartBootloadOperation(siliconId, siliconRev, &blVer, securityKey);
                

        bootloaderEntered = 1;

        appId -= 1; /* 1 and 2 are legal inputs to function. 0 and 1 are valid for bootloader component */
        if (appId > 1)
        {
            appId = INVALID_APP;
        }

        if ((CYRET_SUCCESS == err) && (appId != INVALID_APP))
        {
            /* This will return error if bootloader is for single app */
            err = CyBtldr_GetApplicationStatus(appId, &isValid, &isActive);

            /* Active app can be verified, but not programmed or erased */
            if (CYRET_SUCCESS == err && VERIFY != action && isActive)
            {
                /* This is multi app */
                err = CYRET_ERR_ACTIVE;
            }
        }
    }

    while (CYRET_SUCCESS == err)
    {
        if (g_abort)
        {
            err = CYRET_ABORT;
            break;
        }

        err = CyBtldr_ReadLine(&lineLen, (char *)line);
        if (CYRET_SUCCESS == err)
        {
            err = ProcessDataRow_v0(action, lineLen, line, update);
        }
        else if (CYRET_ERR_EOF == err)
        {
            err = CYRET_SUCCESS;
            break;
        }
    }

    if (err == CYRET_SUCCESS)
    {
        if (PROGRAM == action && INVALID_APP != appId)
        {
            err = CyBtldr_GetApplicationStatus(appId, &isValid, &isActive);

            if (CYRET_SUCCESS == err)
            {
                /* If valid set the active application to what was just programmed */
                /* This is multi app */
                err = (0 == isValid)
                          ? CyBtldr_SetApplicationStatus(appId)
                          : CYRET_ERR_CHECKSUM;
            }
            else if (CYBTLDR_STAT_ERR_CMD == (err ^ (int)CYRET_ERR_BTLDR_MASK))
            {
                /* Single app - restore previous CYRET_SUCCESS */
                err = CYRET_SUCCESS;
            }
        }
        else if (PROGRAM == action || VERIFY == action)
        {
            err = CyBtldr_VerifyApplication();
        }
        CyBtldr_EndBootloadOperation();
    }
    else if (CYRET_ERR_COMM_MASK != (CYRET_ERR_COMM_MASK & err) && bootloaderEntered)
    {
        CyBtldr_EndBootloadOperation();
    }
    return err;
}

int CyBtldr_RunAction(CyBtldr_Action action, const char *file, const uint8_t *securityKey,
                      uint8_t appId, CyBtldr_ProgressUpdate *update)
{
    g_abort = 0;    // Clearing the Global Abort Flag from the Unit
    uint32_t lineLen;   // variable to store Length of Filename 
    uint8_t line[MAX_BUFFER_SIZE * 2]; // 2 hex characters per byte Buffer 
    
    int err;
    uint8_t fileVersion = 0;

    err = CyBtldr_OpenDataFile(file);   // Opening the .cyacd File from the SPIFFS for ESp32
    if (CYRET_SUCCESS == err)
    {
        err = CyBtldr_ReadLine(&lineLen, (char *)line); // --> Passing Empty Buffer to ReadLine Function and lenght

        // ab jab humare pass length, buffer aur SilicID sab aa gaya to check karo wo cyacd file he ya cyacd2
        if (CYRET_SUCCESS == err)
            err = CyBtldr_ParseCyacdFileVersion(file, lineLen, line, &fileVersion); // file --> file ka naam

        if (CYRET_SUCCESS == err)
        {

            err = RunAction_v0(action, lineLen, line, appId, securityKey, update);

        }

        CyBtldr_CloseDataFile();
    }
    

    return err;
}

int CyBtldr_Program(const char *file, const uint8_t *securityKey, uint8_t appId,
                     CyBtldr_ProgressUpdate *update)
{
    return CyBtldr_RunAction(PROGRAM, file, securityKey, appId, update);
}

int CyBtldr_Erase(const char *file, const uint8_t *securityKey, 
                  CyBtldr_ProgressUpdate *update)
{
    return CyBtldr_RunAction(ERASE, file, securityKey, 0,  update);

}

int CyBtldr_Verify(const char *file, const uint8_t *securityKey, 
                   CyBtldr_ProgressUpdate *update)
{
    return CyBtldr_RunAction(VERIFY, file, securityKey, 0, update);
}

int CyBtldr_Abort(void)
{
    g_abort = 1;
    return CYRET_SUCCESS;
}
