/*******************************************************************************
* Copyright 2011-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cybtldr_command.h"

#include "Uart.h"


/* Variable used to store the currently selected packet checksum type */
CyBtldr_ChecksumType CyBtldr_Checksum = SUM_CHECKSUM;

static void fillData16(uint8_t* buf, uint16_t data)
{
    
    buf[0] = (uint8_t)(data);
    buf[1] = (uint8_t)(data >> 8);
}

static void fillData32(uint8_t* buf, uint32_t data)
{
    fillData16(buf, (uint16_t)data);
    fillData16(buf + 2, (uint16_t)(data >> 16));
}

void CyBtldr_SetCheckSumType(CyBtldr_ChecksumType chksumType)
{
    CyBtldr_Checksum = chksumType;
}

static int ParseGenericCmdResult(uint8_t* cmdBuf, uint32_t dataSize, uint32_t expectedSize, uint8_t* status)
{
    int err = CYRET_SUCCESS;
    uint32_t cmdSize = dataSize + BASE_CMD_SIZE;
    *status = cmdBuf[1];
    if (cmdSize != expectedSize)
        err = CYRET_ERR_LENGTH;
    else if (*status != CYRET_SUCCESS)
        err = CYRET_ERR_BTLDR_MASK | (*status);
    else if (cmdBuf[0] != CMD_START || cmdBuf[2] != ((uint8_t)dataSize) || cmdBuf[3] != ((uint8_t)(dataSize >> 8)) || cmdBuf[cmdSize - 1] != CMD_STOP)
        err = CYRET_ERR_DATA;
    return err;
}

int CyBtldr_ParseDefaultCmdResult(uint8_t* cmdBuf, uint32_t cmdSize, uint8_t* status)
{
    return ParseGenericCmdResult(cmdBuf, 0, cmdSize, status);
}


static int CreateCmd(uint8_t* cmdBuf, uint32_t cmdSize, uint8_t cmdCode)	// Creates Send Command for Data(Update / .cyacd file content)
{
    uint16_t checksum;

    // Start of command packet
    cmdBuf[0] = CMD_START;

    // Command code
    cmdBuf[1] = cmdCode;

    // Fill command data size (excluding start, code, and checksum)
    fillData16(cmdBuf + 2, (uint16_t)cmdSize - BASE_CMD_SIZE); // (cmdbuf + 2, 0x07)

    // Compute and fill checksum
    checksum = CyBtldr_ComputeChecksum16bit(cmdBuf, cmdSize - 3);   // (cmdbuf, 0x04)
    
    fillData16(cmdBuf + cmdSize - 3, checksum); // (cmdbuf, 0x04)

    // End of command packet
    cmdBuf[cmdSize - 1] = CMD_STOP;

    return CYRET_SUCCESS;
}




int CyBtldr_CreateEnterBootLoaderCmd(uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize, const uint8_t* securityKeyBuf)
{
    
    const uint16_t RESULT_DATA_SIZE = 8;
    const uint16_t BOOTLOADER_SECURITY_KEY_SIZE = 6;
    uint16_t commandDataSize;
    uint16_t i;

    // Calculate the size of command data based on the presence of a security key
    if (securityKeyBuf != NULL)
        commandDataSize = BOOTLOADER_SECURITY_KEY_SIZE;
    else
        commandDataSize = 0;    // apne case me sirf yahi hoga because we're not using any security key

    // response packet me mujhko 15 size ka data chahiye hoga.
    *resSize = BASE_CMD_SIZE + RESULT_DATA_SIZE;

    // command packet ka size 0x07 yahi pe define kar le rahe he taaki wo command size galat na jaye Touch Pe.
    *cmdSize = BASE_CMD_SIZE + commandDataSize;

    // Fill the command data with the security key, if provided
    for (i = 0; i < commandDataSize; i++){
        cmdBuf[i + 4] = securityKeyBuf[i];
    }

    // Create the command packet using the CreateCmd function
    return CreateCmd(cmdBuf, *cmdSize, CMD_ENTER_BOOTLOADER);   // (inbuf, 0x07, 0x38)
}




int CyBtldr_ParseEnterBootLoaderCmdResult(uint8_t* cmdBuf, uint32_t cmdSize, uint32_t* siliconId, uint8_t* siliconRev, uint32_t* blVersion, uint8_t* status)
{
    const uint32_t RESULT_DATA_SIZE = 8;

    // Parse the generic command result, extracting status
    int err = ParseGenericCmdResult(cmdBuf, RESULT_DATA_SIZE, cmdSize, status);

    // If the generic parsing is successful, extract additional data
    if (CYRET_SUCCESS == err)
    {
        // Extract silicon ID, silicon revision, and bootloader version from the response packet
        *siliconId = (cmdBuf[7] << 24) | (cmdBuf[6] << 16) | (cmdBuf[5] << 8) | cmdBuf[4];
        *siliconRev = cmdBuf[8];
        *blVersion = (cmdBuf[11] << 16) | (cmdBuf[10] << 8) | cmdBuf[9];
    }

    return err;
}


int CyBtldr_CreateExitBootLoaderCmd(uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    *cmdSize = BASE_CMD_SIZE;
    *resSize = BASE_CMD_SIZE;
    return CreateCmd(cmdBuf, *cmdSize, CMD_EXIT_BOOTLOADER);
}

int CyBtldr_CreateProgramRowCmd(uint8_t arrayId, uint16_t rowNum, uint8_t* buf, uint16_t size, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    const uint16_t COMMAND_DATA_SIZE = 3;
    uint32_t i;

    // Set the expected response size and calculate the total command size
    *resSize = BASE_CMD_SIZE;
    *cmdSize = BASE_CMD_SIZE + COMMAND_DATA_SIZE + size;

    // Fill the command buffer with array ID, row number, and data
    cmdBuf[4] = arrayId;
    
    /*cmdBuf + 5 points to the memory location 5 positions after the beginning
    of the cmdBuf array. In C, array indices start from 0, so cmdBuf + 5 effectively
    points to the 6th element of the array. */
    fillData16(cmdBuf + 5, rowNum);
    
    // Fill the command buffer with the data to be programmed.
    // The loop iterates over each byte of the data buffer (buf) and copies it to
    // the command buffer starting from position cmdBuf + 7.
    for (i = 0; i < size; i++)
        cmdBuf[i + 7] = buf[i];

    // Create the command packet using the generic command creation function
    return CreateCmd(cmdBuf, *cmdSize, CMD_PROGRAM_ROW);
}


int CyBtldr_ParseProgramRowCmdResult(uint8_t* cmdBuf, uint32_t cmdSize, uint8_t* status)
{
    return CyBtldr_ParseDefaultCmdResult(cmdBuf, cmdSize, status);
}

int CyBtldr_CreateVerifyRowCmd(uint8_t arrayId, uint16_t rowNum, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    const uint16_t RESULT_DATA_SIZE = 1;
    const uint16_t COMMAND_DATA_SIZE = 3;
    *resSize = BASE_CMD_SIZE + RESULT_DATA_SIZE;
    *cmdSize = BASE_CMD_SIZE + COMMAND_DATA_SIZE;

    cmdBuf[4] = arrayId;
    fillData16(cmdBuf + 5, rowNum);
    return CreateCmd(cmdBuf, *cmdSize, CMD_GET_ROW_CHECKSUM);
}

int CyBtldr_ParseVerifyRowCmdResult(uint8_t* cmdBuf, uint32_t cmdSize, uint8_t* checksum, uint8_t* status)
{
    const uint16_t RESULT_DATA_SIZE = 1;
    int err = ParseGenericCmdResult(cmdBuf, RESULT_DATA_SIZE, cmdSize, status);
    if (CYRET_SUCCESS == err)
    {
        *checksum = cmdBuf[4];
    }
    return err;
}

int CyBtldr_CreateEraseRowCmd(uint8_t arrayId, uint16_t rowNum, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    const uint16_t COMMAND_DATA_SIZE = 3;
    *resSize = BASE_CMD_SIZE;
    *cmdSize = BASE_CMD_SIZE + COMMAND_DATA_SIZE;

    cmdBuf[4] = arrayId;
    fillData16(cmdBuf + 5, rowNum);
    return CreateCmd(cmdBuf, *cmdSize, CMD_ERASE_ROW);
}

int CyBtldr_ParseEraseRowCmdResult(uint8_t* cmdBuf, uint32_t cmdSize, uint8_t* status)
{
    return CyBtldr_ParseDefaultCmdResult(cmdBuf, cmdSize, status);
}

int CyBtldr_CreateVerifyChecksumCmd(uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    const uint16_t RESULT_DATA_SIZE = 1;
    *cmdSize = BASE_CMD_SIZE;
    *resSize = BASE_CMD_SIZE + RESULT_DATA_SIZE;
    return CreateCmd(cmdBuf, *cmdSize, CMD_VERIFY_CHECKSUM);
}

int CyBtldr_ParseVerifyChecksumCmdResult(uint8_t* cmdBuf, uint32_t cmdSize, uint8_t* checksumValid, uint8_t* status)
{
    const uint16_t RESULT_DATA_SIZE = 1;
    int err = ParseGenericCmdResult(cmdBuf, RESULT_DATA_SIZE, cmdSize, status);
    if (CYRET_SUCCESS == err)
    {
        *checksumValid = cmdBuf[4];
    }
    return err;
}

int CyBtldr_CreateGetFlashSizeCmd(uint8_t arrayId, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    const uint16_t RESULT_DATA_SIZE = 4;
    const uint16_t COMMAND_DATA_SIZE = 1;
    *resSize = BASE_CMD_SIZE + RESULT_DATA_SIZE;
    *cmdSize = BASE_CMD_SIZE + COMMAND_DATA_SIZE;

    cmdBuf[4] = arrayId;
    return CreateCmd(cmdBuf, *cmdSize, CMD_GET_FLASH_SIZE);
}

int CyBtldr_ParseGetFlashSizeCmdResult(uint8_t* cmdBuf, uint32_t cmdSize, uint16_t* startRow, uint16_t* endRow, uint8_t* status)
{
    const uint32_t RESULT_DATA_SIZE = 4;
    int err = ParseGenericCmdResult(cmdBuf, RESULT_DATA_SIZE, cmdSize, status);
    if (CYRET_SUCCESS == err)
    {
        *startRow = (cmdBuf[5] << 8) | cmdBuf[4];
        *endRow = (cmdBuf[7] << 8) | cmdBuf[6];
    }
    return err;
}

int CyBtldr_CreateSendDataCmd(uint8_t* buf, uint16_t size, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    uint16_t i;
    *resSize = BASE_CMD_SIZE;
    *cmdSize = size + BASE_CMD_SIZE;

    for (i = 0; i < size; i++)
        cmdBuf[i + 4] = buf[i];
    return CreateCmd(cmdBuf, *cmdSize, CMD_SEND_DATA);
}

int CyBtldr_ParseSendDataCmdResult(uint8_t* cmdBuf, uint32_t cmdSize, uint8_t* status)
{
    return CyBtldr_ParseDefaultCmdResult(cmdBuf, cmdSize, status);
}

int CyBtldr_CreateSyncBootLoaderCmd(uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    *cmdSize = BASE_CMD_SIZE;
    *resSize = BASE_CMD_SIZE;

    return CreateCmd(cmdBuf, *cmdSize, CMD_SYNC);
}

int CyBtldr_CreateGetAppStatusCmd(uint8_t appId, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    const uint16_t RESULT_DATA_SIZE = 2;
    const uint16_t COMMAND_DATA_SIZE = 1;
    *resSize = BASE_CMD_SIZE + RESULT_DATA_SIZE;
    *cmdSize = BASE_CMD_SIZE + COMMAND_DATA_SIZE;

    cmdBuf[4] = appId;
    return CreateCmd(cmdBuf, *cmdSize, CMD_GET_APP_STATUS);
}

int CyBtldr_ParseGetAppStatusCmdResult(uint8_t* cmdBuf, uint32_t cmdSize, uint8_t* isValid, uint8_t* isActive, uint8_t* status)
{
    const uint16_t RESULT_DATA_SIZE = 2;
    int err = ParseGenericCmdResult(cmdBuf, RESULT_DATA_SIZE, cmdSize, status);
    if (CYRET_SUCCESS == err)
    {
        *isValid = cmdBuf[4];
        *isActive = cmdBuf[5];
    }
    return err;
}

int CyBtldr_CreateSetActiveAppCmd(uint8_t appId, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    const uint16_t COMMAND_DATA_SIZE = 1;
    *resSize = BASE_CMD_SIZE;
    *cmdSize = BASE_CMD_SIZE + COMMAND_DATA_SIZE;

    cmdBuf[4] = appId;
    return CreateCmd(cmdBuf, *cmdSize, CMD_SET_ACTIVE_APP);
}

int CyBtldr_ParseSetActiveAppCmdResult(uint8_t* cmdBuf, uint32_t cmdSize, uint8_t* status)
{
    return CyBtldr_ParseDefaultCmdResult(cmdBuf, cmdSize, status);
}

int CyBtldr_CreateProgramDataCmd(uint32_t address, uint32_t chksum, uint8_t* buf, uint16_t size, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    const uint16_t COMMAND_DATA_SIZE = 8;
    uint16_t i;
    *resSize = BASE_CMD_SIZE;
    *cmdSize = BASE_CMD_SIZE + COMMAND_DATA_SIZE + size;

    fillData32(cmdBuf + 4, address);
    fillData32(cmdBuf + 8, chksum);
    for (i = 0; i < size; i++)
        cmdBuf[i + 4 + COMMAND_DATA_SIZE] = buf[i];
    return  CreateCmd(cmdBuf, *cmdSize, CMD_PROGRAM_DATA);
}

int CyBtldr_CreateVerifyDataCmd(uint32_t address, uint32_t chksum, uint8_t* buf, uint16_t size, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    const uint16_t COMMAND_DATA_SIZE = 8;
    uint16_t i;
    *resSize = BASE_CMD_SIZE;
    *cmdSize = BASE_CMD_SIZE + COMMAND_DATA_SIZE + size;

    fillData32(cmdBuf + 4, address);
    fillData32(cmdBuf + 8, chksum);
    for (i = 0; i < size; i++)
        cmdBuf[i + 4 + COMMAND_DATA_SIZE] = buf[i];
    return CreateCmd(cmdBuf, *cmdSize, CMD_VERIFY_DATA);
}

int CyBtldr_CreateEraseDataCmd(uint32_t address, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    const uint16_t COMMAND_DATA_SIZE = 4;
    *resSize = BASE_CMD_SIZE;
    *cmdSize = BASE_CMD_SIZE + COMMAND_DATA_SIZE;

    fillData32(cmdBuf + 4, address);
    return CreateCmd(cmdBuf, *cmdSize, CMD_ERASE_DATA);
}

int CyBtldr_CreateVerifyChecksumCmd_v1(uint8_t appId, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    const uint16_t COMMAND_DATA_SIZE = 1;
    *resSize = BASE_CMD_SIZE + 1;
    *cmdSize = BASE_CMD_SIZE + COMMAND_DATA_SIZE;

    cmdBuf[4] = appId;
    return CreateCmd(cmdBuf, *cmdSize, CMD_VERIFY_CHECKSUM);
}

int CyBtldr_CreateSetApplicationMetadataCmd(uint8_t appID, uint8_t* buf, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    uint32_t i;
    const uint16_t BTDLR_SDK_METADATA_SIZE = 8;
    const uint16_t COMMAND_DATA_SIZE = BTDLR_SDK_METADATA_SIZE + 1;
    *resSize = BASE_CMD_SIZE;
    *cmdSize = BASE_CMD_SIZE + COMMAND_DATA_SIZE;

    cmdBuf[4] = appID;
    for (i = 0; i < BTDLR_SDK_METADATA_SIZE; i++)
    {
        cmdBuf[5 + i] = buf[i]; // Storing 
    }
    return CreateCmd(cmdBuf, *cmdSize, CMD_SET_METADATA);
}

int CyBtldr_CreateSetEncryptionInitialVectorCmd(uint8_t* buf, uint16_t size, uint8_t* cmdBuf, uint32_t* cmdSize, uint32_t* resSize)
{
    uint32_t i;
    *resSize = BASE_CMD_SIZE;
    *cmdSize = BASE_CMD_SIZE + size;
    for (i = 0; i < size; i++)
    {
        cmdBuf[4 + i] = buf[i];
    }
    return CreateCmd(cmdBuf, *cmdSize, CMD_SET_EIV);
}

//Try to parse a packet to determine its validity, if valid then return set the status param to the packet's status.
//Used to generate useful error messages. return 1 on success 0 otherwise.
int CyBtldr_TryParseParketStatus(uint8_t* packet, int packetSize, uint8_t* status)
{
    uint16_t dataSize;
    uint16_t readChecksum;
    uint16_t computedChecksum;
    if (packet == NULL || packetSize < BASE_CMD_SIZE || packet[0] != CMD_START)
        return CYBTLDR_STAT_ERR_UNK;
    *status = packet[1];
    dataSize = packet[2] | (packet[3] << 8);

    readChecksum = packet[dataSize + 4] | (packet[dataSize + 5] << 8);
    computedChecksum = CyBtldr_ComputeChecksum16bit(packet, BASE_CMD_SIZE + dataSize - 3);
    
    if (packet[dataSize + BASE_CMD_SIZE - 1] != CMD_STOP || readChecksum != computedChecksum)
        return CYBTLDR_STAT_ERR_UNK;
    return CYRET_SUCCESS;
}


uint16_t CyBtldr_ComputeChecksum16bit(uint8_t* buf, uint32_t size) {
    uint16_t checksum = 0;
 
    // Iterate through each byte in the buffer
    for (uint8_t i = 0; i < size; i++) {
        checksum += buf[i];  // Add the byte to the checksum
    }

    checksum = ~checksum;
    checksum = checksum + 1;
 
    return checksum;
}
