/*******************************************************************************
* Copyright 2011-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include <string.h>
#include <ctype.h>
#include "cybtldr_parse.h"

#include "Uart.h"

/* Pointer to the *.cyacd file containing the data that is to be read */
static FILE* dataFile;

// Function to parse a 2-byte value in big-endian format
static uint16_t parse2ByteValueBigEndian(uint8_t* buf)
{
    return ((uint16_t)(buf[0] << 8)) | ((uint16_t)buf[1]);
}

// Function to parse a 4-byte value in big-endian format
static uint32_t parse4ByteValueBigEndian(uint8_t* buf)
{
    return (((uint32_t)parse2ByteValueBigEndian(buf)) << 16) | ((uint32_t)parse2ByteValueBigEndian(buf + 2));
}

// Function to parse a 2-byte value in little-endian format
static uint16_t parse2ByteValueLittleEndian(uint8_t* buf)
{
    return ((uint16_t)buf[0]) | (((uint16_t)buf[1]) << 8);
}

// Function to parse a 4-byte value in little-endian format
static uint32_t parse4ByteValueLittleEndian(uint8_t* buf)
{
    return ((uint32_t)parse2ByteValueLittleEndian(buf)) | (((uint32_t)parse2ByteValueLittleEndian(buf + 2)) << 16);
}

// Function to check if two strings are equal, ignoring case
static int stringEqualIgnoreCase(const char* str1, const char* str2)
{
    while (((*str1) != 0 && (*str2) != 0))
    {
        if (tolower(*str1) != tolower(*str2))
        {
            return 0;
        }
        str1++;
        str2++;
    }
    return (tolower(*str1) == tolower(*str2));
}

// Function to convert a hex character to its decimal equivalent
uint8_t CyBtldr_FromHex(char value)
{
    // Check if the character is in the range '0' to '9'
    if ('0' <= value && value <= '9')
        return (uint8_t)(value - '0');  // Convert and return decimal equivalent

    // Check if the character is in the range 'a' to 'f'
    if ('a' <= value && value <= 'f')
        return (uint8_t)(10 + value - 'a');  // Convert and return decimal equivalent with offset

    // Check if the character is in the range 'A' to 'F'
    if ('A' <= value && value <= 'F')
        return (uint8_t)(10 + value - 'A');  // Convert and return decimal equivalent with offset

    // Return 0 if the character is not a valid hex character
    return 0;
}


// Function to convert ASCII hex representation to HexaDecimal
int CyBtldr_FromAscii(uint32_t bufSize, uint8_t* buffer, uint16_t* rowSize, uint8_t* rowData)
{
    volatile uint16_t i;      // Loop variable
    int err = CYRET_SUCCESS;  // Initialize error variable to success

    // Check if the buffer size is an even number of bytes
    if (bufSize & 1)
        err = CYRET_ERR_LENGTH;  // Set error to length error if not an even number of bytes
    else
    {
        // Loop to convert ASCII hex pairs to binary representation
        for (i = 0; i < bufSize / 2; i++)
        {
            // Convert two ASCII hex characters to a single binary byte
            rowData[i] = (CyBtldr_FromHex(buffer[i * 2]) << 4) | CyBtldr_FromHex(buffer[i * 2 + 1]);
        }

    }
        *rowSize = i;  // Store the number of converted bytes in the provided rowSize variable

    return err;  // Return the error status
}


// Function to read a line from the data file, ignoring comments
int CyBtldr_ReadLine(uint32_t* size, char* buffer)
{
    int err = CYRET_SUCCESS;  // Initialize error variable to success
    uint32_t len;             // Variable to store the length of the read line

    // Loop to continue reading lines until a non-comment line is encountered or an error occurs
    do
    {
        len = 0;  // Initialize the length variable to zero

        // Check if the "filename(Directory)" is open and not at the end of the file
        if (NULL != dataFile && !feof(dataFile))
        {
            // Read a line from the ".cyacd" into the Empty buffer, with a maximum size of MAX_BUFFER_SIZE * 2 = 1500
            if (NULL != fgets(buffer, MAX_BUFFER_SIZE * 2, dataFile))
            {
                // Calculate the length of the read line
                len = strlen(buffer);

                // Trim newline characters from the end of the line : "\n" --> New Line Char "\r"
                while (len > 0 && ('\n' == buffer[len - 1] || '\r' == buffer[len - 1]))
                    --len;

            }
            else
                err = CYRET_ERR_EOF;  // Set error to end-of-file if fgets fails
        }
        else
            err = CYRET_ERR_FILE;  // Set error to file error if dataFile is not open or at end-of-file
    } while (err == CYRET_SUCCESS && buffer[0] == '#');  // Continue loop if no error and line is a comment

    *size = len;  // Store the size of already read file that we have these  much of new Line
    return err;   // Return the error status
}


// *.cyacd  file ko open karne k liye file handling lagaya
int CyBtldr_OpenDataFile(const char* file)
{
    dataFile = fopen(file, "r");
    return (NULL == dataFile)   // agar file ki khopdi Sajag Bhai ki tarah khaali hogi to 
        ? CYRET_ERR_FILE        //--> Ye milega "err" me
        : CYRET_SUCCESS;        // warna Success
}

// *.cyacd file se uska "version" parse kar rha he ye wala function
int CyBtldr_ParseCyacdFileVersion(const char* fileName, uint32_t bufSize, uint8_t* header, uint8_t* version)
{
    // File extension ko check karo, agar extension ".cyacd" he to --> version 0 hoga
    int index = strlen(fileName);   // Saving the length of opened .cyacd file name
    int err = CYRET_SUCCESS;    // Setting the Success

    if (bufSize == 0)   // Case for handling file exception
    {
        err = CYRET_ERR_FILE;
    }

    // Find the last '.' character in the filename
    while (CYRET_SUCCESS == err && fileName[--index] != '.')
    {
        if (index == 0)
            err = CYRET_ERR_FILE;
    }

    // Determine the file version based on the file extension
    if (stringEqualIgnoreCase(fileName + index, ".cyacd2"))
    {
        if (bufSize < 2)
            err = CYRET_ERR_FILE;

        // .cyacd2 file stores version information in the first byte of the file header.
        if (CYRET_SUCCESS == err)
        {
            *version = CyBtldr_FromHex(header[0]) << 4 | CyBtldr_FromHex(header[1]);
            if (*version == 0)
                err = CYRET_ERR_DATA;
        }
    }
    else if (stringEqualIgnoreCase(fileName + index, ".cyacd"))
    {
        // .cyacd file does not contain version information
        *version = 0;   // Therefore setting zero to Version 
    }
    else
    {
        err = CYRET_ERR_FILE;
    }
    return err;
}

/*  header ka information Parse karta he *.cyacd file se. The header information
*   is stored as the first line, so this method should only be called once, 
*   and only immediately after calling OpenDataFile and reading the first line. */
int CyBtldr_ParseHeader(uint32_t bufSize, uint8_t* buffer, uint32_t* siliconId, uint8_t* siliconRev, uint8_t* chksum)
{
    int err = CYRET_SUCCESS;

    uint16_t rowSize = 0;       // variable to store the 1st row size of .cyacd file
    uint8_t rowData[MAX_BUFFER_SIZE];   // New Buffer to store the silicondID and siliconRev from .cuacd

    if (CYRET_SUCCESS == err)
    {
        err = CyBtldr_FromAscii(bufSize, buffer, &rowSize, rowData);    // Converting .cyacd ASCII into the required format to send via UART to my Touch Panel
    }

    if (CYRET_SUCCESS == err)
    {
        if (rowSize > 5)
            *chksum = rowData[5];
        else
            *chksum = 0;

        if (rowSize > 4)
        {
            *siliconId = parse4ByteValueBigEndian(rowData);
            *siliconRev = rowData[4];
        }
        else
            err = CYRET_ERR_LENGTH;
    }
    int i = 0;
    return err;
}


// Function to parse row data from the data buffer
int CyBtldr_ParseRowData(uint32_t bufSize, uint8_t* buffer, uint8_t* arrayId, uint16_t* rowNum, uint8_t* rowData, uint16_t* size, uint8_t* checksum)
{
    const uint16_t MIN_SIZE = 6; // Minimum size of a valid row data: 1-array, 2-addr, 2-size, 1-checksum
    const int DATA_OFFSET = 5;    // Offset to the start of actual data in the hex buffer

    unsigned int i;
    uint16_t hexSize;
    uint8_t hexData[MAX_BUFFER_SIZE];
    int err = CYRET_SUCCESS;

    // Check if the buffer size is less than the minimum required size
    if (bufSize <= MIN_SIZE)
        err = CYRET_ERR_LENGTH;
    else if (buffer[0] == ':') // Check if the line starts with ':', indicating valid hex data
    {
        // Convert ASCII hex representation to binary data
        err = CyBtldr_FromAscii(bufSize - 1, &buffer[1], &hexSize, hexData);

        if (err == CYRET_SUCCESS)
        {
            // Extract information from the hex data
            *arrayId = hexData[0];                         // Byte 0: Array ID
            *rowNum = parse2ByteValueBigEndian(hexData + 1); // Bytes 1-2: Row Number
            *size = parse2ByteValueBigEndian(hexData + 3);   // Bytes 3-4: Size of data
            *checksum = (hexData[hexSize - 1]);             // Last byte: Checksum

            // Check if the calculated size matches the actual size of the hex data
            if ((*size + MIN_SIZE) == hexSize)
            {
                // Copy the actual data from hex buffer to the output rowData
                for (i = 0; i < *size; i++)
                {
                    rowData[i] = (hexData[DATA_OFFSET + i]);
                }
            }
            else
                err = CYRET_ERR_DATA; // Invalid data size
        }
    }
    else
        err = CYRET_ERR_CMD; // Invalid command

    return err;
}

// Function to close the *.cyacd data file
int CyBtldr_CloseDataFile(void)
{
    int err = 0;

    if (NULL != dataFile)
    {
        err = fclose(dataFile);
        dataFile = NULL;
    }

    return (0 == err)
        ? CYRET_SUCCESS
        : CYRET_ERR_FILE;
}

